{
COPYRIGHT 1995 STUART LUNN
}

unit Unit1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, clipBrd, about;

type
  TfrmCapture = class(TForm)
    Panel1: TPanel;
    cmdRegion: TButton;
    cmdScreen: TButton;
    ScrollBox1: TScrollBox;
    imgCapture: TImage;
    cmdCopy: TButton;
    cmdInvert: TButton;
    cmdExit: TButton;
    cmdAbout: TButton;
    cmdSaveAs: TButton;
    dlgSaveAs: TSaveDialog;
    chkLeave: TCheckBox;
    chkRemoveHints: TCheckBox;
    procedure cmdRegionClick(Sender: TObject);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure cmdScreenClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cmdCopyClick(Sender: TObject);
    procedure cmdInvertClick(Sender: TObject);
    procedure cmdAboutClick(Sender: TObject);
    procedure cmdExitClick(Sender: TObject);
    procedure cmdSaveAsClick(Sender: TObject);
    procedure chkRemoveHintsClick(Sender: TObject);

  private
    { Private declarations }
    pointStart : TPoint;
    pointEnd : TPoint;
    iCaptureMode : integer;
    bLeftDown : boolean;

    procedure captureIt;
    procedure drawDeskBox;

  public
    { Public declarations }
  end;

var
  frmCapture: TfrmCapture;

const
  CAPTURE_PART = 1;
  CAPTURE_SCREEN = 2;
  CAPTURE_OFF = 3;

  crFINECROSS = 1;

implementation

{$R *.DFM}

{pull in the cross hair from xhair1.res}
{$R xhair1.res}

{=========================================================}
procedure TfrmCapture.captureIt;
{to perfrom the screen copying/bliting}

var
  hWinDesk : integer;
  hDCDesk : integer;

  pointDeskStart : TPoint;

  iWidth : integer;
  iHeight : integer;

  x : integer;
  y : integer;

  bOK : boolean;
  iTemp : integer;

  bitTemp : TBitmap;	{ temporary variable to hold the bitmap }

  iCoord : integer;

begin

  {check capture mode before setting coords}
  case iCaptureMode of
  CAPTURE_PART: begin
  {o selected part of the screen}
    {ensure got coords in ascending order}
    if pointStart.x > pointEnd.x then begin
      iCoord := pointStart.x;
      pointStart.x := pointEnd.x;
      pointEnd.x := iCoord;
    end;

    if pointStart.y> pointEnd.y then begin
      iCoord := pointStart.y;
      pointStart.y := pointEnd.y;
      pointEnd.y := iCoord;
    end;

    {set start, width, height to highlighted region}
    pointDeskStart := clientToScreen(pointStart);
    iWidth := pointEnd.x - pointStart.x;
    iHeight := pointEnd.y - pointStart.y;
    end;

  CAPTURE_SCREEN: begin
  {o capture whole screen}
    {set start, width, height to full screen}
    pointDeskStart.x := 0;
    pointDeskStart.y := 0;
    iWidth := screen.width;
    iHeight := screen.height;
    end;

  else showMessage('Programming error, captureIt(), no SELECT');

  end;

  {origin for the copy is always 0,0}
  x := 0;
  y := 0;

  {create temp bitmap to hold the capture}
  bitTemp := TBitmap.Create;
  bitTemp.Width := iWidth;
  bitTemp.Height := iHeight;

  {get handles to desktop}
  hWinDesk := GetDesktopWindow;
  hDCDesk := GetWindowDC(hWinDesk);

  {bit blt the desktop region into bitmap and assign to image control}
  bOK := BitBlt(bitTemp.canvas.handle, x, y, iWidth,
         iHeight, hDCDesk, pointDeskStart.x, pointDeskStart.y, SRCCOPY);
  imgCapture.picture.assign(bitTemp);

  {free bitmap and release desktop dc}
  bitTemp.free;
  iTemp := ReleaseDC(hWinDesk, hDCDesk);

end;

{=========================================================}
procedure TfrmCapture.cmdRegionClick(Sender: TObject);
{to set regional capture in motion}
var
II,JJ : longint;

begin

  {set the capture mode to any part}
  iCaptureMode := CAPTURE_PART;

  {set this form as the capture control}
  SetCaptureControl(frmCapture);

  {set the fine cross hair cursor}
  screen.cursor := crFINECROSS;

  {check if to make form invisible, if so do it}
  if not chkLeave.checked then
    frmCapture.hide;

end;

{=========================================================}
procedure TfrmCapture.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
{to permit regional screen capture if requested,
 works with FormMouseDrag and FormMouseUp}

begin

  {check capture, if not then exit}
  if iCaptureMode = CAPTURE_OFF then exit;

  {check left button pressed}
  bLeftDown := false;
  if button = mbLeft then begin
    {o yes, set to id that left mouse button down - Shift dont work in mouseMove}
    bLeftDown := true;

    {record the current coords as start and end points}
    {initialise end cos rubber banding otherwise just need on mouse up}
    pointStart.X := X;
    pointStart.Y := Y;

    pointEnd.X := X;
    pointEnd.Y := Y;
  end;

end;

{=========================}
procedure TfrmCapture.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
{to show rubber band so user can see what's
 being captured if regional screen capture requested,
 works with FormMouseDown and FormMouseUp}

begin

  {check capture, if not then exit}
  if iCaptureMode = CAPTURE_OFF then exit;

  {check got left button down, if not exit}
  if not bLeftDown then exit;

  {remove old box}
  drawDeskBox;

  {update end point and draw new box}
  pointEnd.x := X;
  pointEnd.y := Y;
  drawDeskBox;

end;

{=========================================================}
procedure TfrmCapture.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
{to permit regional screen capture if requested,
 works with FormMouseDown and FormMouseDrag}

begin

  {check capture, if not then exit}
  if iCaptureMode = CAPTURE_OFF then exit;

  {reset cursor}
  screen.cursor := crDefault;

  {remove last drawn box}
  drawDeskBox;

  {set to id that left mouse button not down - Shift dont work in mouseMove}
  bLeftDown := false;

  {switch off the capture to the form}
  SetCaptureControl(NIL);

  {record the current coords as end point}
  pointEnd.X := X;
  pointEnd.Y := Y;

  {capture the marked area of the screen}
  captureIt;

  {set the capture mode off again so form doesnt get mouse clicks}
  iCaptureMode := CAPTURE_OFF;

  {ensure form visible}
  frmCapture.Visible := True;

end;

{======================================}
procedure TfrmCapture.cmdScreenClick(Sender: TObject);
{to capture the whole screen}

begin

  {set the capture mode to the whole screen}
  iCaptureMode := CAPTURE_SCREEN;

  {check if to hide form, if so do it}
  if not chkLeave.checked then begin
    hide;
    application.processmessages;
  end;

  {capture the screen}
  captureIt;

  {ensure form visible}
  frmCapture.Visible := True;

end;

{=========================}
procedure TfrmCapture.FormCreate(Sender: TObject);
{to do initialisations}

var
  szCur : array[0..20] of char;

begin

  {load cross hair cursor resource CURSOR_1}
  strPCopy(szCur, 'CURSOR_1');
  Screen.Cursors[crFINECROSS] := LoadCursor(hInstance, szCur);

  {position and size the image}
  imgCapture.top := 0;
  imgCapture.left := 0;
  imgCapture.width := 0;
  imgCapture.height := 0;

  {set the form mouse down/up off}
  iCaptureMode := CAPTURE_OFF;
end;

{==========================}
procedure TfrmCapture.cmdCopyClick(Sender: TObject);
{to copy current image to the clipboard}

begin
  clipboard.assign(imgCapture.picture);
end;

{===========================}
procedure TfrmCapture.cmdInvertClick(Sender: TObject);
{to invert the colours of the captured image}

var
  iWidth : integer;
  iHeight : integer;

  hDC : integer;
  x : integer;
  y : integer;

  bOK : boolean;

begin

  {origin always at 0,0 for invert}
  x := 0;
  y := 0;

  {get the dimensions}
  iWidth := imgCapture.Width;
  iHeight := imgCapture.Height;

  {set the dc and blit to invert}
  hDC := imgCapture.canvas.handle;
  bOK := BitBlt(hDC, x, y, iWidth, iHeight, hDC, x, y, DSTINVERT);

  {then ensure updated}
  imgCapture.refresh;

end;

{======================}
procedure TfrmCapture.drawDeskBox;
{to draw a rectangle from client sp to ep points
 translated to desktop coords
 using the current drawing settings}

var
  hWinDesk : integer;
  hDCDesk : integer;

  pointS : TPoint;
  pointE : TPoint;

  iTemp : integer;

begin

  {handles to desktop}
  hWinDesk := GetDesktopWindow;
  hDCDesk := GetWindowDC(hWinDesk);

  {set to invert the lines}
  SetROP2(hDCDesk, R2_NOT);

  {translate coords to desktop and draw box using lines}
  pointS := ClientToScreen(pointStart);  {forms method not the api fn}
  pointE := ClientToScreen(pointEnd);

  {move to the start and draw the box}
  moveto(hDCDesk, pointS.x, pointS.y);
  lineto(hDCDesk, pointS.x, pointE.y);
  lineto(hDCDesk, pointE.x, pointE.y);
  lineto(hDCDesk, pointE.x, pointS.y);
  lineto(hDCDesk, pointS.x, pointS.y);

  {and release the dc}
  iTemp := ReleaseDC(hWinDesk, hDCDesk);

end;

{======================}
procedure TfrmCapture.cmdAboutClick(Sender: TObject);
{to show the about box}
begin
  frmAbout.showModal;
end;

{======================}
procedure TfrmCapture.cmdExitClick(Sender: TObject);
{to close the form/app}
begin
  close;
end;

{======================}
procedure TfrmCapture.cmdSaveAsClick(Sender: TObject);
{to save the pic to a file}

begin
  if dlgSaveAs.execute then begin
    imgCapture.picture.saveToFile(dlgSaveAs.filename);
  end;
end;

{===============}
procedure TfrmCapture.chkRemoveHintsClick(Sender: TObject);
{to set/remove hints}
begin
  showHint := not showHint;
end;

end.

