{
 Designer: Craig Ward, 100554.2072@compuserve.com
 Date:     8/11/95
 Version:  1.31

 Function: Table filter component.

 Calling:  There are three fields that must be set:
            [1] Table               - table name to filter on
            [2] Field               - the field name to filter on
            [3] Filter              - the actual filter expression
            [4] FieldIndex          - the index of the field in the table
           There are two procedures to be called:
            [1] SetFilter           - add and activate filter
            [2] RemoveFilter        - deactivate and drop filter

 Notes:    Big thanks goto:
            Mark Erbaugh (71370.1475@compuserve.com)
            Mike Toms    (miketoms@calladin.demon.co.uk)
            Peter Below  (100113.1101@compuserve.com)

 Problems:
           [1] At present the component can not filter on currency, time
               or timestamp fields. I am working on this!
           [2] The developer has 256 characters (less 2 for nulls, and 20 or less
               for a field name) in which to store a filter expression. You could
               bump this up if you wished (see the line: literals: array[0..255] of char.
           [3] The filter can only work on one table, and only one field at a time,
               though you can build progressive filters (ie: filter on one field, then
               call the component to filter on another field, and so on). Wow, it
               makes you appreciate the filter function in Paradox, which
               can filter on a whole set of tables and fields all at one time!
*******************************************************************************}
unit Filter;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, DBiTypes, DBiProcs, DBiErrs;

type
  TTabFilter = class(TComponent)
  private
    { Private declarations }
    FTable: TTable;                              {store's tablename}
    FField: string;                              {store's fieldname}
    FFilter: string;                             {store's filter}
    FIndex: integer;                             {store's field index}
    procedure SetTablename(value: TTable);
    procedure SetField(value: string);
    procedure SetFilterExpression(value: string);
    procedure SetFieldIndex(value: integer);
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure SetFilter;                         {set filter on table}
    procedure RemoveFilter;                      {remove filter from table}
  published
    { Published declarations }
    property Table: TTable read FTable write SetTablename;
    property Field: string read FField write SetField;
    property Filter: string read FFilter write SetFilterExpression;
    property FieldIndex: integer read FIndex write SetFieldIndex;
  end;

procedure Register;

{******************************************************************************}
implementation

{register component}
procedure Register;
begin
 RegisterComponents('Samples', [TTabFilter]);
end;

{set table field}
procedure TTabFilter.SetTablename(value: TTable);
begin
 if value <> FTable then
  begin
   FTable := value;
  end;
end;


{set fieldname field}
procedure TTabFilter.SetField(value: string);
begin
 if value <> FField then
  begin
   FField := value
  end;
end;

{set filter field}
procedure TTabFilter.SetFilterExpression(value: string);
begin
 if value <> FFilter then
  begin
   FFilter := value
  end;
end;

{set field index}
procedure TTabFilter.SetFieldIndex(value: integer);
begin
 if value <> FIndex then
  begin
   FIndex := value;
  end;
end;


{set filter on table}
procedure TTabFilter.SetFilter;
{custom type - stores filter expression and components}
type
  TmyFilter = record
   Expr: CANExpr;
   Nodes: array[0..2] of CANNode;
   literals: array[0..255] of char;
end;
{******************************************************}
var
  myFilter: TmyFilter;         {instance of custom type}
  fldName,
  fldFilter: pChar;            {fieldname and filter char's}
  dbResult: DBiResult;         {result from BDE}
  hFilter: hDBiFilter;         {handle to filter}
  si: integer;                 {stores filter for smallint field type}
  li, liT: longint;            {stores filter for longint field type}
  ex: extended;                {stores filter for date,time,float and currency fields}
  dt: TDateTime;               {used in date\time fields}
  dl: double;                  {used for timeStamp fields}
  yr, mh, dy,
  hr, mn, sc, ms: word;        {used for date\time fields}
  {******************************************************}
begin
 {find field type}
 case FTable.Fields[FIndex].DataType of
   ftString:
    myFilter.nodes[2].canConst.iType := fldZSTRING;
   ftCurrency:
    myFilter.nodes[2].canConst.iType := fldstMONEY;
   ftDate:
    myFilter.nodes[2].canConst.iType := fldDATE;
   ftTime:
    myFilter.nodes[2].canConst.iType := fldTIME;
   ftFloat:
    myFilter.nodes[2].canConst.iType := fldFLOAT;
   ftInteger:
    myFilter.nodes[2].canConst.iType := fldINT32;
   ftSmallInt:
    myFilter.nodes[2].canConst.iType := fldINT16;
   ftDateTime:
    myFilter.nodes[2].canConst.iType := fldTIMESTAMP;
 end;
 {******************************************************}
 try
  {allocate resources and copy}
  fldName := StrAlloc(256);
  StrPCopy(fldName,FField);
  StrCopy(myFilter.Literals,fldName);
  {*****************************************************}
  {switch to account for field type - setting filter}
  case FTable.Fields[FIndex].DataType of
    {string}
    ftString:
     begin
      try
       fldFilter := StrAlloc(256);
       StrPCopy(fldFilter,FFilter);
       {***}
       StrCat(myFilter.Literals,'Z');                 {catenate temp char value}
       StrCat(myFilter.Literals,fldFilter);           {add filter to literals}
       myFilter.Literals[StrLen(fldName)] := #0;      {replace temp with null}
       StrCat(myFilter.Literals,#0);                  {add null to end of filter}
      finally
       StrDispose(fldFilter);                         {dispose of filter pChar}
      end;
     end;
    {long int}
    ftInteger:
     begin
      li := StrToInt(FFilter);
      move(li,myFilter.Literals[StrLen(fldName)+1],sizeOf(li));
     end;
    {small int}
    ftSmallInt:
     begin
      si := StrToInt(FFilter);
      move(si,myFilter.Literals[StrLen(fldName)+1],sizeOf(si));
     end;
    {float}
    ftFloat:
     begin
      dl := StrToFloat(FFilter);
      move(dl,myFilter.Literals[StrLen(fldName)+1],sizeOf(dl));
     end;
    {date}
    ftDate:
     begin
      dt := StrToDate(FFilter);
      DecodeDate(dt,yr,mh,dy);
      dbResult := DBiDateEncode(mh,dy,integer(yr),li);
      if dbResult = DBIERR_NONE then
       move(li,myFilter.Literals[StrLen(fldName)+1],sizeOf(li))
      else
       begin
        messageDlg('Could not encode date.',mtWarning,[mbOK],0);
        exit;
       end;
     end;
    {time}
    ftTime:
     begin
      messageDlg('Can not place filters on time fields',mtWarning,[mbOK],0);
      exit;
      {dt := StrToTime(FFilter);
      DecodeTime(dt,hr,mn,sc,ms);
      dbResult := DBiTimeEncode(hr,mn,ms,li);
      if dbResult = DBiERR_NONE then
       move(li,myFilter.Literals[StrLen(fldName)+1],sizeOf(li))
      else
       begin
        messageDlg('Could not encode time.',mtWarning,[mbOK],0);
        exit;
       end;}
     end;
    ftDateTime:
     begin
      messageDlg('Can not place filters on TimeStamp fields',mtWarning,[mbOK],0);
      exit;
      {dt := StrToDateTime(FFilter);
      li := trunc(dt);
      DecodeTime(dt,hr,mn,sc,ms);
      dbResult := DBiTimeEncode(hr,mn,ms,liT);
      dbResult := DBiTimeStampEncode(li,liT,dl);
      move(dl,myFilter.Literals[StrLen(fldName)+1],sizeOf(dl));}
     end;
    {money}
    ftCurrency:
     begin
      messageDlg('Can not place filters on currency fields.',mtWarning,[mbOK],0);
      exit;
      {dl := StrToFloat(FFilter);
      move(dl,myFilter.Literals[StrLen(fldName)+1],sizeOf(dl));}
     end;
  end;
  {*****************************************************}
  {set the CANExpr field of the filter expression}
   myFilter.Expr.iVer := 1;
   myFilter.Expr.iTotalSize := sizeOf(myFilter);
   myFilter.Expr.iNodes := 3;
   myFilter.Expr.iNodeStart := sizeOf(CANExpr);
   myFilter.Expr.iLiteralStart := sizeOf(CANExpr) + 3 * sizeOf(CANNode);
  {set the Nodes field of the filter expression}
   {first part of array}
    myFilter.nodes[0].canBinary.NodeClass  := nodeBinary;
    myFilter.nodes[0].canBinary.canOP := canEQ;
    myFilter.nodes[0].canBinary.iOperand1 := sizeOf(CANNode);
    myFilter.nodes[0].canBinary.iOperand2 := 2 * sizeOf(CANNode);
   {second part of array}
    myFilter.nodes[1].canField.nodeClass := nodeField;
    myFilter.nodes[1].canField.canOP := canField2;
    myFilter.nodes[1].canField.iFieldNum := 0;
    myFilter.nodes[1].canField.iNameOffset := 0;
   {third part of array}
    myFilter.nodes[2].canConst.nodeClass := nodeConst;
    myFilter.nodes[2].canConst.canOP := canCONST2;
    myFilter.nodes[2].canConst.iSize := 3;
    myFilter.nodes[2].canConst.iOffset := StrLen(fldName) + 1;
   {run filter}
   dbResult := DBiAddFilter(FTable.handle,1,1,False,addr(myFilter),nil,hFilter);
   if dbResult = DBIERR_NONE then
    begin
     dbResult := DBiActivateFilter(FTable.handle,hFilter);
     if dbResult = DBIERR_NONE then
      FTable.first
     else
      messageDlg('Could not activate filter',mtWarning,[mbOK],0);
    end
   else
    messageDlg('Could not set filter',mtWarning,[mbOK],0);
 finally
  StrDispose(fldName);    {free resources allocated to fieldname}
  end;
end;



{remove filter}
procedure TTabFilter.RemoveFilter;
var
  dbResult: DBiResult; {result from BDE}
  hFilter: hDBiFilter;  {handle to filter}
begin
 dbResult := DBiDeactivateFilter(FTable.handle,nil); {if filter handle is null, then all table filters are deactivated}
 if dbResult = DBIERR_NONE then
  begin
   dbResult := DBiDropFilter(FTable.handle,nil);     {if filter handle is null, then all table filters are dropped}
   if dbResult = DBIERR_NONE then
    begin
     FTable.First;
    end
   else
    messageDlg('Could not dispose of filter',mtWarning,[mbOK],0);
  end
 else
  messageDlg('Could not deactivate filter',mtWarning,[mbOK],0);
end;


end.
